//-----------------------------------------------------------------------------
// Project            	GeometryKit
// Class		G3DQuaternion
// Creator            	Philippe C.D. Robert
// Maintainer         	Philippe C.D. Robert
// Creation Date      	Thu Sep  9 11:05:08 CEST 1999
//
// Copyright (c) Philippe C.D. Robert
//
// The GeometryKit is free software; you can redistribute it and/or modify it 
// under the terms of the GNU LGPL Version 2 as published by the Free 
// Software Foundation
//
// $Id: G3DQuaternion.h,v 1.6 2001/01/05 20:50:48 robert Exp $
//-----------------------------------------------------------------------------

/*
  Quaternion routines are partially based on the paper from
  Nick Bobick, GameDeveloper Vol.2, Issue 26. They are not entirely correct, 
  though, so I fixed them...
 */

#ifndef __G3DQuaternion_h_INCLUDE
#define __G3DQuaternion_h_INCLUDE

#import <Foundation/Foundation.h>
#import <GeometryKit/G3DTuple4f.h>

@class G3DMatrix3f;
@class G3DMatrix4f;
@class G3DVector3f;

@interface G3DQuaternion : G3DTuple4f
{
}

//-----------------------------------------------------------------------------
// class methods
//-----------------------------------------------------------------------------

+ (void)initialize;

+ (G3DQuaternion *)quaternion;
// Returns an autoreleased unity quaternion

+ (G3DQuaternion *)quaternionWithElements:(float *)values;
+ (G3DQuaternion *)quaternionWithQuaternion:(G3DQuaternion *)aTuple;
// these methods invoke the corresponding methods from G3DTuple4f!

+ (G3DQuaternion *)quaternionWithMatrix4f:(G3DMatrix4f *)aMatrix;
// Matrix is a rotation matrix

+ (G3DQuaternion *)quaternionWithEulerRep:(float *)ev;

//-----------------------------------------------------------------------------
// init and free
//-----------------------------------------------------------------------------

- (id)init;
  // Returns an autoreleased unity quaternion, Designated initialiser!

- (id)initWithMatrix4f:(G3DMatrix4f *)aMatrix;
  // Matrix is a rotation matrix

- (id)initWithEulerRep:(float *)ev;

//-----------------------------------------------------------------------------
// Math
//-----------------------------------------------------------------------------

- (float)norm;
// N = sum(a_i^2)

- (void)conjugate;
// q* = [-v,w]

- (G3DQuaternion *)conjugatedQuaternion;

- (void)invert;
// q^-1 = q*/N(q)

- (G3DQuaternion *)invertedQuaternion;

- (void)normalise;
// q2 = q/L(q)

- (G3DQuaternion *)normalisedQuaternion;

- (void)interpolate:(G3DQuaternion *)quat factor:(float)val;
// Interpolates (SLERP - spherical linear interpolation) between self and quat

- (void)multiplyByQuaternion:(G3DQuaternion *)a;
- (void)multiplyByQuaternion:(G3DQuaternion *)a and:(G3DQuaternion *)b;

- (void)multiplyByInvertedQuaternion:(G3DQuaternion *)a;
- (void)multiplyByQuaternion:(G3DQuaternion *)a andInverted:(G3DQuaternion *)b;

- (void)multiplyByScalar:(float)value;

- (void)addQuaternion:(G3DQuaternion *)aQuat;
- (void)subQuaternion:(G3DQuaternion *)aQuat;

- (void)rotateByAngle:(float)angle axis:(G3DVector3f *)aVec;
// Builds a new quaternion q, multiplies self with q and normalises itself

//-----------------------------------------------------------------------------
// Accessor methods
//-----------------------------------------------------------------------------

- (void)setQuaternionWithMatrix4f:(G3DMatrix4f *)m;

- (G3DMatrix4f *)rotationMatrix4f;
// Returns a 4x4 rotation matrix, based upon self

- (G3DMatrix3f *)rotationMatrix3f;
// Returns a 3x3 rotation matrix, based upon self

- (void)setQuaternion:(G3DQuaternion *)q;
// Invokes [super setValuesWithTuple:q]

- (void)setVector:(G3DVector3f *)vec angle:(float)val;
// Initialises the quaternion from an angle val and an axis vec 

- (G3DTuple4f *)angleAxisRepresentation;
// Returns a vector containing the axis in the first 3 values and the angle in the forth

@end

extern NSString *G3DQuaternionException;

#endif





